/***************************************************************************

  /systems/odyssey2.c

  Driver file to handle emulation of the Odyssey2.

***************************************************************************/

#include "driver.h"
#include "cpu/i8039/i8039.h"
#include "vidhrdw/generic.h"
#include "includes/odyssey2.h"
#include "devices/cartslot.h"
#include "inputx.h"

ADDRESS_MAP_START( odyssey2_mem , ADDRESS_SPACE_PROGRAM, 8)
	AM_RANGE( 0x0000, 0x03FF) AM_ROM
	AM_RANGE( 0x0400, 0x0BFF) AM_RAMBANK(1)
	AM_RANGE( 0x0C00, 0x0FFF) AM_RAMBANK(2)
ADDRESS_MAP_END

ADDRESS_MAP_START( odyssey2_io , ADDRESS_SPACE_IO, 8)
	AM_RANGE( 0x00,		 0xff)		AM_READWRITE(	  odyssey2_bus_r, odyssey2_bus_w)
	AM_RANGE( I8039_p1,	 I8039_p1)	AM_READWRITE( odyssey2_getp1, odyssey2_putp1 )
	AM_RANGE( I8039_p2,	 I8039_p2)	AM_READWRITE( odyssey2_getp2, odyssey2_putp2 )
	AM_RANGE( I8039_bus, I8039_bus)	AM_READWRITE( odyssey2_getbus, odyssey2_putbus )
	AM_RANGE( I8039_t1,  I8039_t1)	AM_READ( odyssey2_t1_r )
ADDRESS_MAP_END

INPUT_PORTS_START( odyssey2 )
	PORT_START		/* IN0 */
	PORT_KEY1( 0x01, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_0, CODE_NONE, '0')
	PORT_KEY1( 0x02, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_1, CODE_NONE, '1')
	PORT_KEY1( 0x04, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_2, CODE_NONE, '2')
	PORT_KEY1( 0x08, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_3, CODE_NONE, '3')
	PORT_KEY1( 0x10, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_4, CODE_NONE, '4')
	PORT_KEY1( 0x20, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_5, CODE_NONE, '5')
	PORT_KEY1( 0x40, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_6, CODE_NONE, '6')
	PORT_KEY1( 0x80, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_7, CODE_NONE, '7')
	PORT_START		/* IN1 */
	PORT_KEY1( 0x01, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_8, CODE_NONE, '8')
	PORT_KEY1( 0x02, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_9, CODE_NONE, '9')
	PORT_KEY1( 0x04, IP_ACTIVE_LOW, "?? :",				KEYCODE_F1, CODE_NONE, UCHAR_MAMEKEY(F1))
	PORT_KEY1( 0x08, IP_ACTIVE_LOW, "?? $",				KEYCODE_F2, CODE_NONE, UCHAR_MAMEKEY(F2))
	PORT_KEY1( 0x10, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_SPACE, CODE_NONE, ' ')
	PORT_KEY1( 0x20, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_SLASH, CODE_NONE, '?')
	PORT_KEY1( 0x40, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_L, CODE_NONE, 'L')
	PORT_KEY1( 0x80, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_P, CODE_NONE, 'P')
	PORT_START		/* IN2 */
	PORT_KEY1( 0x01, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_PLUS_PAD, CODE_NONE, '+')
	PORT_KEY1( 0x02, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_W, CODE_NONE, 'W')
	PORT_KEY1( 0x04, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_E, CODE_NONE, 'E')
	PORT_KEY1( 0x08, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_R, CODE_NONE, 'R')
	PORT_KEY1( 0x10, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_T, CODE_NONE, 'T')
	PORT_KEY1( 0x20, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_U, CODE_NONE, 'U')
	PORT_KEY1( 0x40, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_I, CODE_NONE, 'I')
	PORT_KEY1( 0x80, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_O, CODE_NONE, 'O')
	PORT_START		/* IN3 */
	PORT_KEY1( 0x01, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_Q, CODE_NONE, 'Q')
	PORT_KEY1( 0x02, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_S, CODE_NONE, 'S')
	PORT_KEY1( 0x04, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_D, CODE_NONE, 'D')
	PORT_KEY1( 0x08, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_F, CODE_NONE, 'F')
	PORT_KEY1( 0x10, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_G, CODE_NONE, 'G')
	PORT_KEY1( 0x20, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_H, CODE_NONE, 'H')
	PORT_KEY1( 0x40, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_J, CODE_NONE, 'J')
	PORT_KEY1( 0x80, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_K, CODE_NONE, 'K')
	PORT_START		/* IN4 */
	PORT_KEY1( 0x01, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_A, CODE_NONE, 'A')
	PORT_KEY1( 0x02, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_Z, CODE_NONE, 'Z')
	PORT_KEY1( 0x04, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_X, CODE_NONE, 'X')
	PORT_KEY1( 0x08, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_C, CODE_NONE, 'C')
	PORT_KEY1( 0x10, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_V, CODE_NONE, 'V')
	PORT_KEY1( 0x20, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_B, CODE_NONE, 'B')
	PORT_KEY1( 0x40, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_M, CODE_NONE, 'M')
	PORT_KEY1( 0x80, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_STOP, CODE_NONE, '.')
	PORT_START		/* IN5 */
	PORT_KEY1( 0x01, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_MINUS, CODE_NONE, '-')
	PORT_KEY1( 0x02, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_ASTERISK, CODE_NONE, '*')
	PORT_KEY1( 0x04, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_SLASH_PAD, CODE_NONE, '/')
	PORT_KEY1( 0x08, IP_ACTIVE_LOW, IP_NAME_DEFAULT,	KEYCODE_EQUALS, CODE_NONE, '=')
	PORT_KEY1( 0x10, IP_ACTIVE_LOW, DEF_STR( Yes ),		KEYCODE_Y, CODE_NONE, 'Y')
	PORT_KEY1( 0x20, IP_ACTIVE_LOW, DEF_STR( No ),		KEYCODE_N, CODE_NONE, 'N')
	PORT_KEY1( 0x40, IP_ACTIVE_LOW, "CLR",				KEYCODE_BACKSPACE, CODE_NONE, 8)
	PORT_KEY1( 0x80, IP_ACTIVE_LOW, "ENT",				KEYCODE_ENTER, CODE_NONE, '\r')
	PORT_START		/* IN6 */
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP)		PORT_PLAYER(1)
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT)	PORT_PLAYER(1)
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN)	PORT_PLAYER(1)
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT)	PORT_PLAYER(1)
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1)			PORT_PLAYER(1)
	PORT_BIT ( 0xe0, 0xe0,	 IPT_UNUSED )
	PORT_START		/* IN7 */
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP)		PORT_PLAYER(2)
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT)	PORT_PLAYER(2)
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN)	PORT_PLAYER(2)
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT)	PORT_PLAYER(2)
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1)			PORT_PLAYER(2)
	PORT_BIT ( 0xe0, 0xe0,	 IPT_UNUSED )
INPUT_PORTS_END

static struct GfxLayout odyssey2_graphicslayout =
{
	8,1,
	256,                                    /* 256 characters */
	1,                      /* 1 bits per pixel */
	{ 0 },                  /* no bitplanes; 1 bit per pixel */
	/* x offsets */
	{ 
	0,
	1,
	2,
	3,
	4,
	5,
	6,
	7,
	},
	/* y offsets */
	{ 0 },
	1*8
};


static struct GfxLayout odyssey2_spritelayout =
{
	8,1,
	256,                                    /* 256 characters */
	1,                      /* 1 bits per pixel */
	{ 0 },                  /* no bitplanes; 1 bit per pixel */
	/* x offsets */
	{ 
	7,6,5,4,3,2,1,0
	},
	/* y offsets */
	{ 0 },
	1*8
};

static struct GfxDecodeInfo odyssey2_gfxdecodeinfo[] =
{
    { REGION_GFX1, 0x0000, &odyssey2_graphicslayout,                     0, 2 },
    { REGION_GFX1, 0x0000, &odyssey2_spritelayout,                     0, 2 },
    { -1 } /* end of array */
};

static MACHINE_DRIVER_START( odyssey2 )
	/* basic machine hardware */
	MDRV_CPU_ADD(I8048, 1790000/5)         /* 1.79 MHz */
	MDRV_CPU_PROGRAM_MAP(odyssey2_mem, 0)
	MDRV_CPU_IO_MAP(odyssey2_io, 0)
	MDRV_CPU_VBLANK_INT(odyssey2_line, 262)
	MDRV_FRAMES_PER_SECOND(60)
	MDRV_VBLANK_DURATION(DEFAULT_REAL_60HZ_VBLANK_DURATION)
	MDRV_INTERLEAVE(1)

	MDRV_MACHINE_INIT( odyssey2 )

    /* video hardware */
	
	/* NPW 23-Nov-2002 - added VIDEO_PIXEL_ASPECT_RATIO_2_1 on behalf of Dopefish
	 * Justin */
	MDRV_VIDEO_ATTRIBUTES(VIDEO_TYPE_RASTER | VIDEO_PIXEL_ASPECT_RATIO_2_1)

	MDRV_SCREEN_SIZE(320,300)
	MDRV_VISIBLE_AREA(0,160-1,0,240-1)
	MDRV_GFXDECODE( odyssey2_gfxdecodeinfo )
	MDRV_PALETTE_LENGTH(24)
	MDRV_COLORTABLE_LENGTH(2)
	MDRV_PALETTE_INIT( odyssey2 )

	MDRV_VIDEO_START( odyssey2 )
	MDRV_VIDEO_UPDATE( odyssey2 )

	/* sound hardware */
	MDRV_SOUND_ADD(CUSTOM, odyssey2_sound_interface)
MACHINE_DRIVER_END

ROM_START (odyssey2)
    ROM_REGION(0x10000,REGION_CPU1,0)    /* safer for the memory handler/bankswitching??? */
    ROM_LOAD ("o2bios.rom", 0x0000, 0x0400, CRC(8016a315) SHA1(b2e1955d957a475de2411770452eff4ea19f4cee))
    ROM_REGION(0x100, REGION_GFX1, 0)
    ROM_REGION(0x2000, REGION_USER1, 0)
ROM_END

SYSTEM_CONFIG_START(odyssey2)
	CONFIG_DEVICE_CARTSLOT_REQ(1, "bin\0", NULL, NULL, device_load_odyssey2_cart, NULL, NULL, NULL)
SYSTEM_CONFIG_END

/*     YEAR  NAME      PARENT	COMPAT	MACHINE   INPUT     INIT      CONFIG    COMPANY     FULLNAME     FLAGS */
COMPX( 1982, odyssey2, 0,		0,		odyssey2, odyssey2, odyssey2, odyssey2, "Magnavox", "Odyssey 2", GAME_NOT_WORKING | GAME_IMPERFECT_SOUND )
/* philips g7000/videopac */
